@@ -392,9 +392,8 @@ static int print_wowlan_handler(struct nl_msg *msg, void *arg)
 		int rem_match;
 
 		printf(" * wake up on network detection\n");
-		nla_parse(nd, NUM_NL80211_ATTR,
-			  nla_data(trig[NL80211_WOWLAN_TRIG_NET_DETECT]),
-			  nla_len(trig[NL80211_WOWLAN_TRIG_NET_DETECT]), NULL);
+		nla_parse_nested(nd, NL80211_ATTR_MAX,
+				 trig[NL80211_WOWLAN_TRIG_NET_DETECT], NULL);
 
 		if (nd[NL80211_ATTR_SCHED_SCAN_INTERVAL])
 			printf("\tscan interval: %u msecs\n",
@@ -409,9 +408,8 @@ static int print_wowlan_handler(struct nl_msg *msg, void *arg)
 			nla_for_each_nested(match,
 					    nd[NL80211_ATTR_SCHED_SCAN_MATCH],
 					    rem_match) {
-				nla_parse(tb, NUM_NL80211_ATTR, nla_data(match),
-					  nla_len(match),
-					  NULL);
+				nla_parse_nested(tb, NL80211_ATTR_MAX, match,
+						 NULL);
 				printf("\t\tSSID: ");
 				print_ssid_escaped(
 					nla_len(tb[NL80211_SCHED_SCAN_MATCH_ATTR_SSID]),
